#!/sbin/sh
#
#	X Initialisation
#
#	To prevent the start of xdm rename /etc/xdm (eg to /etc/xdm.orig)
#	This also prevents the x login from being allowed.
#
#	To run this machine as an X terminal (contacting a remote X
#	display manager) edit the Xconfig.<memsize>Mbytes file, to
#	change the XDM-HOST field and the AUTHENTICATION (or 
#	NOAUTHENTICATION) field - see the comments in Xconfig.*.
#
#	To run X continuously put a line for a local X server into
#	/etc/xdm/Xservers and prevent the creation of the Xconfig
#	file below (to prevent the x login).
#
#	To prevent the x login remove the file /etc/xdm/Xconfig.
#
#	For more information read the RISC iX X Window System
#	Administrator's Guide.
#
#-----------------------------------------------------------------------
# IMPORTANT
#
#	Xstart assumes that the host name ``loopback'' is equivalent to
#	the ip address 127.0.0.1 - this special address identifies the
#	local machine.  If you use yellow pages, or if the relevant line
#	in the /etc/hosts has been edited to remove ``loopback'' you
#	need to add an XDM-HOST entry to all the /etc/xdm/Xconfig* files
#	which identifies the local host.
#
#-----------------------------------------------------------------------
#
# Test for the existence of /etc/xdm.  If it does exist execute the
# remainder of this file in a sub-shell so that the variables which
# are set up and the PATH change is not exported
#
if [ -d /etc/xdm ]
then
    if (
	#
	# Note that this path setting is made inside () so it does not
	# affect the main /etc/rc file.  All commands started here have
	# /etc/xdm as their working directory.  This means that /etc/xdm
	# cannot be removed/unmounted while any of these commands
	# continue to run.
	#
	PATH=/sbin:/usr/sbin:/usr/ucb:/usr/bin:/usr/bin/X11
	cd /etc/xdm
	#
	# The x login is controlled by the file /etc/xdm/Xconfig, which
	# Xstart reads when starting the X server.  If this file exists
	# and the AUTHOR field is /etc/rc.X it is regenerated here.
	#
	if [ -f Xconfig ]
	then
	    #
	    # NOTE: the runes:-
	    #
	    #	var=`command` || exit 1
	    #
	    # causes the subshell to assign the output of "command" to
	    # the shell variable "var", if the command succeeds (exit status
	    # 0) nothing else happens, if it FAILS the "exit 1" is executed
	    # which causes the subshell itself to exit.  This is used
	    # extensively below - normally the command error output (stream 2)
	    # is also redirected to /dev/console.
	    #
	    author=`xadmin -author 2>/dev/console` || exit 1
	    #
	    # The AUTHOR field must exist and be set to /etc/rc.X
	    #
	    if [ "$author" = /etc/rc.X ]
	    then
		#
		# The file is owned by this shell script but is not
		# deleted yet as this would prevent it being regenerated
		# if any of the following goes wrong.
		#
		# The following runes obtain the memory size of the
		# system.  If this cannot be done an error message will
		# be output on /dev/console and the multi-user boot will
		# exit.
		#
		memsize=`xadmin -memsize 2>/dev/console` || exit 1
		#
		# X is not fully supported on low resolution monitors.  The
		# following deals with this by preventing the creation of
		# /etc/xdm/Xconfig if the vdu type is 0
		#
		vdutype=`xadmin -vdutype 2>/dev/console` || exit 1
		#
		# Remove the following test to support low resolution monitors
		# or replace by ``if false''.
		#
		if [ "$vdutype" = 0 ]
		then
		    (echo -n ' (X not supported on monitor type 0)')	>/dev/console 2>&1
		    exit 0
		else
		    #
		    # Construct the Xconfig file.
		    #
		    rm -f Xconfig.new
		    if (
			#
			# Any failure in the following aborts the file creation
			#
			set -e
			echo '# /etc/xdm/Xconfig'
			echo '#'
			echo '# This is a machine generate file - if you edit it change'
			echo '# the AUTHOR string at the end of the file to prevent the'
			echo '# script /etc/rc.X from regenerating it when the system'
			echo '# goes multi-user.'
			echo '# Derived from' /etc/xdm/Xconfig.${memsize}Mbytes
			echo '# Removing this file will prevent the x login from'
			echo '# being used.'
			echo '#'
			xadmin -vdustring
			echo '#'
			cat /etc/xdm/Xconfig.${memsize}Mbytes
			echo '#'
			echo AUTHOR /etc/rc.X
			exit 0
		    )					>Xconfig.new	2>/dev/console
		    then
			#
			# Rename must be atomic - a valid Xconfig must
			# always exist.  Rely on the kernel to do this.
			#
			if mv -f Xconfig.new Xconfig
			then
			    (echo ' (Xconfig)')				>/dev/console 2>&1
			else
			    exit 1
			fi
		    else
			exit 1
		    fi
		fi
	    fi
	fi
	#
	# Find the display id of this machine.  xadmin uses the (newly
	# created) Xconfig file for this - so -serialNo, -displayID
	# and -class arguments in this file (which will be supplied to
	# the server by Xstart(1)) are taken into account.
	#
	displayid=`xadmin -displayid 2>/dev/console` || exit 1
	(echo -n "X Server Display ID: ${displayid}")			>/dev/console
	#
	# If authentication is being done it is also necessary to edit
	# the authentication key file.  The display id of the server
	# is supplied with a new key; either by adding one if the
	# display id does not exist in the file (or the file does not
	# exist) or by editting the current entry.  The key file which
	# is edited here is /etc/xdm/Xkeys - this is the key file of
	# the local xdm.  If this machine is running as an X terminal
	# the server should use a different key file so that the following
	# changes do not overwrite the key (which is shared with xdm
	# programs on other machines on the network).  We check the
	# server key file anyway to allow a warning about the lack of
	# authentication to be produced.
	#
	keyfile=`xadmin -keyfile 2>/dev/console` || exit 1
	xdmhost=`xadmin -host 2>/dev/console` || exit 1
	(echo -n " XDM host: ${xdmhost}")				>/dev/console
	if [ "$keyfile" = "" ]
	then
	    (echo -n " (not authenticated)")				>/dev/console
	fi
	(echo)								>/dev/console
	#
	# And generate a key...
	#
	key=`xadmin -keygen 2>/dev/console` || exit 1
	#
	# Make sure the key file exists, then edit it.  To make
	# this secure it is essential that the key file is only
	# readable by the super-user.
	#
	if [ ! -f Xkeys ]; then touch Xkeys >/dev/console 2>&1; fi
	rm -f Xkeys.new
	if (
	    set -e
	    touch Xkeys.new
	    chmod 600 Xkeys.new
	    awk "BEGIN{d=\"${displayid}\"; k=\"${key}\"; f=0}"'\
		($1==d){$2=k; f=1} {print $0}\
		END{ if (f==0) print d,k}' <Xkeys >Xkeys.new
	    # Rename atomically
	    mv -f Xkeys.new Xkeys
	    exit 0
	)								>/dev/console 2>&1
	then
	    (echo -n '(Xkeys)')						>/dev/console
	else
	    exit 1
	fi
	#
	# xdm will only be started if an Xservers file also exists -
	# the file may be empty.  The x login will not be disabled
	# by removing the Xservers file; so an X server will start but
	# no login window will appear unless the X server has been
	# configured to query another machine (see Xstart(1)/xadmin(1))
	# Notice that xdm must be started AFTER the Xkeys file has been
	# generated.
	#
	# If you want the X server to run all the time edit
	# /etc/xdm/Xservers to start it up as a local X server - you will
	# also need to disable the x login by removing /etc/xdm/Xconfig.
	#
	if [ `xadmin -vdutype` = 2 ]
	then
		DISP="DisplayManager.servers: :0      local   /usr/bin/X11/Xarm"
	else
		DISP="DisplayManager.servers: :0     local   /usr/bin/X11/Xarm -c4 -scr -c4 -scr -c8"
	fi

	/usr/bin/X11/xdm -xrm "$DISP" 

#	if [ -f Xservers ]
#	then
#	    if /usr/bin/X11/xdm					>/dev/console 2>&1
#	    then
#		(echo -n ' xdm')				>/dev/console 2>&1
#	    else
#		(echo -n ' xdm failed to start')		>/dev/console 2>&1
#		exit 1
#	    fi
#	fi
#	     
	#
	#*******************************************************************
	# To use this machine as an X terminal, EITHER configure Xconfig to
	# start a server which will query the remote machine, OR add a line
	# similar to the following one to start an X server which will
	# contact xdm running on the remote machine.  The latter method will
	# cause the server to run all the time and is therefore not
	# recommended - you will also need to disable the x login by
	# removing /etc/xdm/Xconfig.
	#*******************************************************************
	#
	# Xarm -query <otherhost>	# (not recommended - edit Xconfig instead)
	#
	# The following is REQUIRED - otherwise this sub-shell will exit
	# with the status of the last command (which may well have been
	# the test ([) command in an if expression).
	#
	exit 0
    ); then
	:
    else
	# An error message has already been output
	exit 1
    fi
fi
