
/*
 * Configuration parameters for rc. Suggested defaults are at the bottom
 * of this file (you should probably look at those first to see if your
 * system matches one of them; you can search for the beginning of the
 * defaults section by looking for the string "#ifndef CUSTOM"). If you
 * want to override the suggested defaults, define the macro CUSTOM.
#define CUSTOM
 */

/*
 * (Note that certain default settings redefine this macro)
 * DEFAULTPATH the default search path that rc uses when it is started
 * without either a $PATH or $path environment variable. You must pick
 * something sensible for your system if you don't like the path shown
 * below.
 */
#define DEFAULTPATH "/usr/ucb", "/usr/bin", "/bin", "."

/*
 * Define the macro SYSTEMV if you wish rc to use System V calls (where
 * they exist) in preference to BSD calls
 */
#define SYSTEMV

/*
 * Define the macro NOCMDARG if you do not have /dev/fd or fifos on your
 * system. You may also want to define this if you have broken fifos.
#define NOCMDARG
 */

/*
 * Define TMPDIR if you need to have rc create its fifos in a directory
 * other than /tmp. For example, if you have a Sun with /tmp mounted
 * as a ramdisk (type "tmpfs") then you cannot use fifos in /tmp (sigh).
#define TMPDIR "/var/tmp"
 */

/*
 * Define the macro DEVFD if your system supports /dev/fd.
#define DEVFD
 */

/*
 * Define the macro NOLIMITS if your system does not support Berkeley
 * limits.
#define NOLIMITS
 */

/*
 * Define the macro NOSIGCHLD if your system uses SIGCHLD in the System
 * V way.
#define NOSIGCHLD
 */

/*
 * Define the macro READLINE if you want rc to call GNU readline
 * instead of read(2) on interactive shells.
 */
#define READLINE

/*
 * Define the macro NOEXECVE if your Unix does not interpret #! in the
 * kernel, and uncomment the EXECVE variable in the Makefile.
#define NOEXECVE
 */

/*
 * If you want rc to default to some interpreter for files which don't
 * have a legal #! on the first line, define the macro DEFAULTINTERP.
#define DEFAULTINTERP "/bin/sh"
 */

/*
 * If your /bin/sh (or another program you care about) rejects environment
 * variables with special characters in them (such as ':' or '-'), rc can
 * put out ugly variable names using [_0-9a-zA-Z] that encode the real name;
 * define PROTECT_ENV for this hack. (Known offenders: every sh I have tried;
 * SunOS (silently discards), NeXT (aborts with error), SGI (aborts with
 * error), Ultrix (sh seems to work, sh5 aborts with error))
#define PROTECT_ENV
 */

/*
 * Define the macro NOECHO if you wish to omit rc's echo builtin from the
 * compile.
#define NOECHO
 */


/* Beginning of defaults section: */

#ifndef CUSTOM

/*
 * Suggested settings for Sun, NeXT and sgi (machines here at TAMU):
 */

#ifdef RIX
#define CONFIGURATION "RISC IX"
#define _BSD_TYPES
#define NOSIGCHLD
#endif

#ifdef NeXT		/* Used on NextOS 2.1 */
#define CONFIGURATION "NeXT"
#define	PROTECT_ENV
#define NOCMDARG
#endif

#ifdef sgi		/* Used on Irix 3.3.[12] */
#define CONFIGURATION "IRIX"
#define SYSTEMV
#define NOSIGCHLD
#define	PROTECT_ENV
#undef DEFAULTPATH
#define DEFAULTPATH "/usr/bsd", "/usr/sbin", "/usr/bin", "/bin", "."
/* need BSD types and signals */
#define _BSD_TYPES
#endif

#ifdef SOLARIS /* Tested on Sun Solaris 2.5 for SPARC */
/* enh 1996/02/27 - /usr/xpg4/bin is where the POSIX stuff is (that has
** different behaviour to standard Solaris), /usr/ccs/bin the programming
** utilities that all good men expect on their path. /bin is just a symbolic
** link to /usr/bin, so it's not necessary. */
#define CONFIGURATION "Solaris"
#define PROTECT_ENV
#undef DEFAULTPATH
#define DEFAULTPATH "/usr/xpg4/bin", "/usr/ccs/bin", "/usr/bin", "."
#endif

#ifdef LINUX /* tested on i386 Linux 2.0.11 - jfc */
#define CONFIGURATION "Linux"
#define PROTECT_ENV
#undef DEFAULTPATH
#define DEFAULTPATH "/bin", "/usr/bin/", "/usr/bin/X11", "."
#endif

/* enh 1996/02/28 - changed the name from sun to sunos. Solaris also
** has sun defined (for obvious reasons), but SunOS 5.5 is nothing
** like SunOS 4. Thank goodness. */
#ifdef SUNOS /* Used on SunOS 4.1.1 */
#define CONFIGURATION "SunOS 4"
#define PROTECT_ENV
#undef DEFAULTPATH
#define DEFAULTPATH "/usr/ucb", "/usr/bin", "."
#endif

/*
 * Suggested settings for Ultrix
 */

#ifdef ultrix
#define CONFIGURATION "Ultrix"
#define PROTECT_ENV
#define DEFAULTINTERP "/bin/sh"	/* so /bin/true can work */
#endif

/*
 * Suggested settings for AIX
 */

#ifdef _AIX
#define CONFIGURATION "AIX"
#define PROTECT_ENV
#endif

/*
 * Suggested settings for OSF/1 1.0
 */

#ifdef OSF1
#define CONFIGURATION "OSF/1"
#define PROTECT_ENV
#endif

/*
 * Suggested settings for Unicos XXX
 */

#ifdef cray
#define CONFIGURATION "Unicos"
#define PROTECT_ENV
#define NOLIMITS
#define word _word
#define DEFAULTINTERP "/bin/sh"
#endif

#endif /* CUSTOM */

#ifndef TMPDIR
#define TMPDIR "/tmp"
#endif
